@@ -292,7 +292,7 @@ void              nautilus_canvas_container_start_renaming_selected_item  (Nauti
 									   gboolean                select_all);
 
 /* options */
-NautilusZoomLevel nautilus_canvas_container_get_zoom_level                (NautilusCanvasContainer  *view);
+NautilusCanvasZoomLevel nautilus_canvas_container_get_zoom_level                (NautilusCanvasContainer  *view);
 void              nautilus_canvas_container_set_zoom_level                (NautilusCanvasContainer  *view,
 									   int                     new_zoom_level);
 void              nautilus_canvas_container_set_single_click_mode         (NautilusCanvasContainer  *container,
@@ -315,9 +315,6 @@ void              nautilus_canvas_container_set_margins                   (Nauti
 									   int                     bottom_margin);
 char*             nautilus_canvas_container_get_icon_description          (NautilusCanvasContainer  *container,
 									     NautilusCanvasIconData       *data);
-gboolean          nautilus_canvas_container_get_allow_moves               (NautilusCanvasContainer  *container);
-void              nautilus_canvas_container_set_allow_moves               (NautilusCanvasContainer  *container,
-									   gboolean                allow_moves);
 
 gboolean	  nautilus_canvas_container_is_layout_rtl			(NautilusCanvasContainer  *container);
 gboolean	  nautilus_canvas_container_is_layout_vertical		(NautilusCanvasContainer  *container);
