@@ -35,6 +35,7 @@ struct _NautilusColumnChooserDetails
 	GtkTreeView *view;
 	GtkListStore *store;
 
+	GtkWidget *main_box;
 	GtkWidget *move_up_button;
 	GtkWidget *move_down_button;
 	GtkWidget *use_default_button;
@@ -232,6 +233,9 @@ row_deleted_callback (GtkTreeModel *model,
 	list_changed (NAUTILUS_COLUMN_CHOOSER (user_data));
 }
 
+static void move_up_clicked_callback (GtkWidget *button, gpointer user_data);
+static void move_down_clicked_callback (GtkWidget *button, gpointer user_data);
+
 static void
 add_tree_view (NautilusColumnChooser *chooser)
 {
@@ -296,9 +300,9 @@ add_tree_view (NautilusColumnChooser *chooser)
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 	gtk_widget_show (GTK_WIDGET (scrolled));
-	
+
 	gtk_container_add (GTK_CONTAINER (scrolled), view);
-	gtk_box_pack_start (GTK_BOX (chooser), scrolled, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (chooser->details->main_box), scrolled, TRUE, TRUE, 0);
 }
 
 static void
@@ -309,9 +313,9 @@ move_up_clicked_callback (GtkWidget *button, gpointer user_data)
 	GtkTreeSelection *selection;
 
 	chooser = NAUTILUS_COLUMN_CHOOSER (user_data);
-	
+
 	selection = gtk_tree_view_get_selection (chooser->details->view);
-	
+
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
 		GtkTreePath *path;
 		GtkTreeIter prev;
@@ -337,14 +341,14 @@ move_down_clicked_callback (GtkWidget *button, gpointer user_data)
 	GtkTreeSelection *selection;
 
 	chooser = NAUTILUS_COLUMN_CHOOSER (user_data);
-	
+
 	selection = gtk_tree_view_get_selection (chooser->details->view);
-	
+
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
 		GtkTreeIter next;
 
 		next = iter;
-		
+
 		if (gtk_tree_model_iter_next (GTK_TREE_MODEL (chooser->details->store), &next)) {
 			gtk_list_store_move_after (chooser->details->store,
 						   &iter,
@@ -365,44 +369,59 @@ use_default_clicked_callback (GtkWidget *button, gpointer user_data)
 static void
 add_buttons (NautilusColumnChooser *chooser)
 {
+	GtkWidget *inline_toolbar;
+	GtkStyleContext *style_context;
+	GtkToolItem *tool_item;
 	GtkWidget *box;
-	GtkWidget *separator;
-	
-	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
-	gtk_widget_show (box);
-	
-	chooser->details->move_up_button = gtk_button_new_with_mnemonic (_("Move _Up"));
-	g_signal_connect (chooser->details->move_up_button, 
+
+	inline_toolbar = gtk_toolbar_new ();
+	gtk_widget_show (GTK_WIDGET (inline_toolbar));
+
+	style_context = gtk_widget_get_style_context (GTK_WIDGET (inline_toolbar));
+	gtk_style_context_add_class (style_context, GTK_STYLE_CLASS_INLINE_TOOLBAR);
+	gtk_box_pack_start (GTK_BOX (chooser->details->main_box), inline_toolbar,
+			    FALSE, FALSE, 0);
+
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	tool_item = gtk_tool_item_new ();
+	gtk_container_add (GTK_CONTAINER (tool_item), box);
+	gtk_container_add (GTK_CONTAINER (inline_toolbar), GTK_WIDGET (tool_item));
+
+	chooser->details->move_up_button = gtk_button_new_from_icon_name ("go-up-symbolic",
+									  GTK_ICON_SIZE_SMALL_TOOLBAR);
+	g_signal_connect (chooser->details->move_up_button,
 			  "clicked",  G_CALLBACK (move_up_clicked_callback),
 			  chooser);
-	gtk_widget_show_all (chooser->details->move_up_button);
 	gtk_widget_set_sensitive (chooser->details->move_up_button, FALSE);
-	gtk_box_pack_start (GTK_BOX (box), chooser->details->move_up_button,
-			    FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (box), chooser->details->move_up_button);
 
-	chooser->details->move_down_button = gtk_button_new_with_mnemonic (_("Move Dow_n"));
-	g_signal_connect (chooser->details->move_down_button, 
+	chooser->details->move_down_button = gtk_button_new_from_icon_name ("go-down-symbolic",
+									    GTK_ICON_SIZE_SMALL_TOOLBAR);
+	g_signal_connect (chooser->details->move_down_button,
 			  "clicked",  G_CALLBACK (move_down_clicked_callback),
 			  chooser);
-	gtk_widget_show_all (chooser->details->move_down_button);
 	gtk_widget_set_sensitive (chooser->details->move_down_button, FALSE);
-	gtk_box_pack_start (GTK_BOX (box), chooser->details->move_down_button,
-			    FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (box), chooser->details->move_down_button);
 
-	separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-	gtk_widget_show (separator);
-	gtk_box_pack_start (GTK_BOX (box), separator, FALSE, FALSE, 0);	
+	tool_item = gtk_separator_tool_item_new ();
+	gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (tool_item), FALSE);
+	gtk_tool_item_set_expand (tool_item, TRUE);
+	gtk_container_add (GTK_CONTAINER (inline_toolbar), GTK_WIDGET (tool_item));
 
-	chooser->details->use_default_button = gtk_button_new_with_mnemonic (_("Use De_fault"));
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	tool_item = gtk_tool_item_new ();
+	gtk_container_add (GTK_CONTAINER (tool_item), box);
+	gtk_container_add (GTK_CONTAINER (inline_toolbar), GTK_WIDGET (tool_item));
+
+	chooser->details->use_default_button = gtk_button_new_with_mnemonic (_("Reset to De_fault"));
+	gtk_widget_set_tooltip_text (chooser->details->use_default_button,
+				     _("Replace the current List Columns settings with the default settings"));
 	g_signal_connect (chooser->details->use_default_button, 
 			  "clicked",  G_CALLBACK (use_default_clicked_callback),
 			  chooser);
-	gtk_widget_show (chooser->details->use_default_button);
-	gtk_box_pack_start (GTK_BOX (box), chooser->details->use_default_button,
-			    FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (box), chooser->details->use_default_button);
 
-	gtk_box_pack_start (GTK_BOX (chooser), box,
-			    FALSE, FALSE, 0);
+	gtk_widget_show_all (inline_toolbar);
 }
 
 static void
@@ -471,6 +490,11 @@ nautilus_column_chooser_init (NautilusColumnChooser *chooser)
 		      "orientation", GTK_ORIENTATION_HORIZONTAL,
 		      NULL);
 
+	chooser->details->main_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+	gtk_widget_set_hexpand (chooser->details->main_box, TRUE);
+	gtk_widget_show (chooser->details->main_box);
+	gtk_container_add (GTK_CONTAINER (chooser), chooser->details->main_box);
+
 	add_tree_view (chooser);
 	add_buttons (chooser);
 }
