@@ -16,9 +16,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __NAUTILUS_APPLICATION_H__
@@ -29,6 +27,7 @@
 #include <gtk/gtk.h>
 
 #include "nautilus-bookmark-list.h"
+#include "nautilus-progress-ui-handler.h"
 #include "nautilus-window.h"
 
 #define NAUTILUS_DESKTOP_ICON_VIEW_IID	"OAFIID:Nautilus_File_Manager_Desktop_Canvas_View"
@@ -64,17 +63,17 @@ NautilusApplication * nautilus_application_new (void);
 NautilusWindow *     nautilus_application_create_window (NautilusApplication *application,
 							 GdkScreen           *screen);
 
+void nautilus_application_add_accelerator (GApplication *app,
+					   const gchar  *action_name,
+					   const gchar  *accel);
+
+GList * nautilus_application_get_windows (NautilusApplication *application);
+
 void nautilus_application_open_location (NautilusApplication *application,
 					 GFile *location,
 					 GFile *selection,
 					 const char *startup_id);
 
-void nautilus_application_notify_unmount_show (NautilusApplication *application,
-					       const gchar *message);
-
-void nautilus_application_notify_unmount_done (NautilusApplication *application,
-					       const gchar *message);
-
 NautilusBookmarkList *
      nautilus_application_get_bookmarks  (NautilusApplication *application);
 void nautilus_application_edit_bookmarks (NautilusApplication *application,
@@ -83,4 +82,6 @@ void nautilus_application_edit_bookmarks (NautilusApplication *application,
 GtkWidget * nautilus_application_connect_server (NautilusApplication *application,
 						 NautilusWindow      *window);
 
+NautilusProgressUIHandler * nautilus_application_get_progress_ui_handler (NautilusApplication *application);
+
 #endif /* __NAUTILUS_APPLICATION_H__ */
