@@ -72,10 +72,13 @@ enum
 
 static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 
+typedef gboolean (* SortCriterionMatchFunc) (NautilusFile *file);
+
 typedef struct {
 	const NautilusFileSortType sort_type;
 	const char *metadata_text;
 	const char *action_target_name;
+	SortCriterionMatchFunc match_func;
 } SortCriterion;
 
 typedef enum {
@@ -137,12 +140,14 @@ static const SortCriterion sort_criteria[] = {
 	{
 		NAUTILUS_FILE_SORT_BY_TRASHED_TIME,
 		"trashed",
-		"trash-time"
+		"trash-time",
+		nautilus_file_is_in_trash
 	},
 	{
 		NAUTILUS_FILE_SORT_BY_SEARCH_RELEVANCE,
 		NULL,
 		"search-relevance",
+		nautilus_file_is_in_search
 	}
 };
 
@@ -797,13 +802,15 @@ nautilus_canvas_view_zoom_to_level (NautilusView *view,
 	g_return_if_fail (new_level >= NAUTILUS_CANVAS_ZOOM_LEVEL_SMALL &&
 			  new_level <= NAUTILUS_CANVAS_ZOOM_LEVEL_LARGE);
 
-
 	canvas_view = NAUTILUS_CANVAS_VIEW (view);
 	canvas_container = get_canvas_container (canvas_view);
 	if (nautilus_canvas_container_get_zoom_level (canvas_container) == new_level)
 		return;
 
 	nautilus_canvas_container_set_zoom_level (canvas_container, new_level);
+	g_action_group_change_action_state (nautilus_view_get_action_group (view),
+					    "zoom-to-level", g_variant_new_int32 (new_level));
+
 	nautilus_view_update_toolbar_menus (view);
 }
 
@@ -823,8 +830,6 @@ nautilus_canvas_view_bump_zoom_level (NautilusView *view, int zoom_increment)
 	    new_level <= NAUTILUS_CANVAS_ZOOM_LEVEL_LARGE) {
 		nautilus_canvas_view_zoom_to_level (view, new_level);
 	}
-
-	nautilus_view_update_toolbar_menus (view);
 }
 
 static void
@@ -961,6 +966,8 @@ action_zoom_to_level (GSimpleAction *action,
 	view = NAUTILUS_VIEW (user_data);
 	zoom_level = g_variant_get_int32 (state);
 	nautilus_canvas_view_zoom_to_level (view, zoom_level);
+
+	g_simple_action_set_state (G_SIMPLE_ACTION (action), state);
 }
 
 static void
@@ -1022,6 +1029,34 @@ const GActionEntry canvas_view_entries[] = {
 	{ "zoom-to-level", NULL, NULL, "1", action_zoom_to_level }
 };
 
+static void
+update_sort_action_state_hint (NautilusCanvasView *canvas_view)
+{
+	NautilusFile *file;
+	GVariantBuilder builder;
+	GActionGroup *action_group;
+	GAction *action;
+	GVariant *state_hint;
+	gint idx;
+
+	file = nautilus_view_get_directory_as_file (NAUTILUS_VIEW (canvas_view));
+	g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
+
+	for (idx = 0; idx < G_N_ELEMENTS (sort_criteria); idx++) {
+		if (sort_criteria[idx].match_func == NULL ||
+		    (file != NULL && sort_criteria[idx].match_func (file)))
+			g_variant_builder_add (&builder, "s", sort_criteria[idx].action_target_name);
+	}
+
+	state_hint = g_variant_builder_end (&builder);
+
+	action_group = nautilus_view_get_action_group (NAUTILUS_VIEW (canvas_view));
+	action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "sort");
+	g_simple_action_set_state_hint (G_SIMPLE_ACTION (action), state_hint);
+
+	g_variant_unref (state_hint);
+}
+
 static void
 nautilus_canvas_view_update_actions_state (NautilusView *view)
 {
@@ -1052,34 +1087,8 @@ nautilus_canvas_view_update_actions_state (NautilusView *view)
 		keep_aligned = nautilus_canvas_container_is_keep_aligned (get_canvas_container (canvas_view));
 		g_action_change_state (action, g_variant_new_boolean (keep_aligned));
 	}
-}
 
-static void
-nautilus_canvas_view_update_toolbar_menus (NautilusView *view)
-{
-	NautilusFile *file;
-	NautilusToolbar *toolbar;
-	NautilusCanvasContainer *canvas_container;
-	gint zoom_level;
-
-	NAUTILUS_VIEW_CLASS (nautilus_canvas_view_parent_class)->update_toolbar_menus (view);
-
-	toolbar = NAUTILUS_TOOLBAR (nautilus_window_get_toolbar (nautilus_view_get_window (view)));
-
-	nautilus_toolbar_show_sort_menu (toolbar);
-
-	file = nautilus_view_get_directory_as_file (NAUTILUS_VIEW (view));
-	if (file != NULL && nautilus_file_is_in_trash (file))
-		nautilus_toolbar_show_sort_trash_time (toolbar);
-
-	if (file != NULL && nautilus_file_is_in_search (file))
-		nautilus_toolbar_show_sort_search_relevance (toolbar);
-
-	canvas_container = get_canvas_container (NAUTILUS_CANVAS_VIEW (view));
-	zoom_level = nautilus_canvas_container_get_zoom_level (canvas_container);
-
-	nautilus_toolbar_view_menu_widget_set_zoom_level (toolbar,
-							  (gdouble) (zoom_level));
+	update_sort_action_state_hint (canvas_view);
 }
 
 static void
@@ -1861,7 +1870,6 @@ nautilus_canvas_view_class_init (NautilusCanvasViewClass *klass)
 	nautilus_view_class->invert_selection = nautilus_canvas_view_invert_selection;
 	nautilus_view_class->compare_files = compare_files;
         nautilus_view_class->click_policy_changed = nautilus_canvas_view_click_policy_changed;
-	nautilus_view_class->update_toolbar_menus = nautilus_canvas_view_update_toolbar_menus;
 	nautilus_view_class->update_actions_state = nautilus_canvas_view_update_actions_state;
         nautilus_view_class->sort_directories_first_changed = nautilus_canvas_view_sort_directories_first_changed;
         nautilus_view_class->start_renaming_file = nautilus_canvas_view_start_renaming_file;
