@@ -24,7 +24,6 @@
 #include <config.h>
 #include "nautilus-desktop-window.h"
 #include "nautilus-window-private.h"
-#include "nautilus-actions.h"
 
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
@@ -33,6 +32,7 @@
 #include <glib/gi18n.h>
 
 #include <eel/eel-vfs-extensions.h>
+#include <libnautilus-private/nautilus-desktop-link-monitor.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-icon-names.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
@@ -41,6 +41,8 @@ struct NautilusDesktopWindowDetails {
 	gulong size_changed_id;
 
 	gboolean loaded;
+
+	GtkWidget *desktop_selection;
 };
 
 G_DEFINE_TYPE (NautilusDesktopWindow, nautilus_desktop_window, 
@@ -62,46 +64,111 @@ nautilus_desktop_window_update_directory (NautilusDesktopWindow *window)
 }
 
 static void
-nautilus_desktop_window_constructed (GObject *obj)
+nautilus_desktop_window_finalize (GObject *obj)
 {
-	GtkActionGroup *action_group;
-	GtkAction *action;
-	AtkObject *accessible;
-	NautilusDesktopWindow *window = NAUTILUS_DESKTOP_WINDOW (obj);
-	NautilusWindow *nwindow = NAUTILUS_WINDOW (obj);
+	nautilus_desktop_link_monitor_shutdown ();
 
-	G_OBJECT_CLASS (nautilus_desktop_window_parent_class)->constructed (obj);
-
-	action_group = nautilus_window_get_main_action_group (nwindow);
-
-	/* Don't allow close action on desktop */
-	action = gtk_action_group_get_action (action_group,
-					      NAUTILUS_ACTION_CLOSE);
-	gtk_action_set_sensitive (action, FALSE);
-
-	/* Don't allow new tab on desktop */
-	action = gtk_action_group_get_action (action_group,
-					      NAUTILUS_ACTION_NEW_TAB);
-	gtk_action_set_sensitive (action, FALSE);
-
-	/* Don't allow search on desktop */
-	action = gtk_action_group_get_action (action_group,
-					      NAUTILUS_ACTION_SEARCH);
-	gtk_action_set_sensitive (action, FALSE);
-
-	/* Set the accessible name so that it doesn't inherit the cryptic desktop URI. */
-	accessible = gtk_widget_get_accessible (GTK_WIDGET (window));
-
-	if (accessible) {
-		atk_object_set_name (accessible, _("Desktop"));
-	}
+	G_OBJECT_CLASS (nautilus_desktop_window_parent_class)->finalize (obj);
 }
 
 static void
-nautilus_desktop_window_init (NautilusDesktopWindow *window)
+nautilus_desktop_window_init_actions (NautilusDesktopWindow *window)
 {
-	window->details = G_TYPE_INSTANCE_GET_PRIVATE (window, NAUTILUS_TYPE_DESKTOP_WINDOW,
-						       NautilusDesktopWindowDetails);
+	GAction *action;
+
+	/* Don't allow close action on desktop */
+	action = g_action_map_lookup_action (G_ACTION_MAP (window),
+					     "close-current-view");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
+
+	/* Don't allow new tab on desktop */
+	action = g_action_map_lookup_action (G_ACTION_MAP (window),
+					     "new-tab");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
+
+	/* Don't allow search on desktop */
+	action = g_action_map_lookup_action (G_ACTION_MAP (window),
+					     "toggle-search");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
+}
+
+static void
+selection_get_cb (GtkWidget          *widget,
+		  GtkSelectionData   *selection_data,
+		  guint               info,
+		  guint               time)
+{
+	/* No extra targets atm */
+}
+
+static gboolean
+selection_clear_event_cb (GtkWidget	        *widget,
+			  GdkEventSelection     *event,
+			  NautilusDesktopWindow *window)
+{
+	gtk_widget_destroy (GTK_WIDGET (window));
+
+	return TRUE;
+}
+
+static void
+nautilus_desktop_window_init_selection (NautilusDesktopWindow *window)
+{
+	char selection_name[32];
+	GdkAtom selection_atom;
+	Window selection_owner;
+	GdkDisplay *display;
+	GtkWidget *selection_widget;
+	GdkScreen *screen;
+
+	screen = gdk_screen_get_default ();
+
+	g_snprintf (selection_name, sizeof (selection_name),
+		    "_NET_DESKTOP_MANAGER_S%d", gdk_screen_get_number (screen));
+	selection_atom = gdk_atom_intern (selection_name, FALSE);
+	display = gdk_screen_get_display (screen);
+
+	selection_owner = XGetSelectionOwner (GDK_DISPLAY_XDISPLAY (display),
+					      gdk_x11_atom_to_xatom_for_display (display,
+										 selection_atom));
+	if (selection_owner != None) {
+		g_critical ("Another desktop manager in use; desktop window won't be created");
+		return;
+	}
+
+	selection_widget = gtk_invisible_new_for_screen (screen);
+	/* We need this for gdk_x11_get_server_time() */
+	gtk_widget_add_events (selection_widget, GDK_PROPERTY_CHANGE_MASK);
+
+	if (!gtk_selection_owner_set_for_display (display,
+						  selection_widget,
+						  selection_atom,
+						  gdk_x11_get_server_time (gtk_widget_get_window (selection_widget)))) {
+		gtk_widget_destroy (selection_widget);
+		g_critical ("Can't set ourselves as selection owner for desktop manager; "
+			    "desktop window won't be created");
+		return;
+	}
+
+	g_signal_connect (selection_widget, "selection-get",
+			  G_CALLBACK (selection_get_cb), window);
+	g_signal_connect (selection_widget, "selection-clear-event",
+			  G_CALLBACK (selection_clear_event_cb), window);
+
+	window->details->desktop_selection = selection_widget;
+}
+
+static void
+nautilus_desktop_window_constructed (GObject *obj)
+{
+	AtkObject *accessible;
+	NautilusDesktopWindow *window = NAUTILUS_DESKTOP_WINDOW (obj);
+        GdkRGBA transparent = {0, 0, 0, 0};
+
+	G_OBJECT_CLASS (nautilus_desktop_window_parent_class)->constructed (obj);
+
+	/* Initialize the desktop link monitor singleton */
+	nautilus_desktop_link_monitor_get ();
 
 	gtk_window_move (GTK_WINDOW (window), 0, 0);
 
@@ -115,6 +182,40 @@ nautilus_desktop_window_init (NautilusDesktopWindow *window)
 
 	g_object_set_data (G_OBJECT (window), "is_desktop_window", 
 			   GINT_TO_POINTER (1));
+
+	nautilus_desktop_window_init_selection (window);
+	nautilus_desktop_window_init_actions (window);
+
+	/* Set the accessible name so that it doesn't inherit the cryptic desktop URI. */
+	accessible = gtk_widget_get_accessible (GTK_WIDGET (window));
+
+	if (accessible) {
+		atk_object_set_name (accessible, _("Desktop"));
+	}
+
+	/* Special sawmill setting */
+	gtk_window_set_wmclass (GTK_WINDOW (window), "desktop_window", "Nautilus");
+
+	/* Point window at the desktop folder.
+	 * Note that nautilus_desktop_window_init is too early to do this.
+	 */
+	nautilus_desktop_window_update_directory (window);
+        gtk_widget_override_background_color (GTK_WIDGET (window), 0, &transparent);
+
+	/* We realize it immediately so that the NAUTILUS_DESKTOP_WINDOW_ID
+	 * property is set so gnome-settings-daemon doesn't try to set
+	 * background. And we do a gdk_flush() to be sure X gets it.
+	 */
+	gtk_widget_realize (GTK_WIDGET (window));
+	gdk_flush ();
+
+}
+
+static void
+nautilus_desktop_window_init (NautilusDesktopWindow *window)
+{
+	window->details = G_TYPE_INSTANCE_GET_PRIVATE (window, NAUTILUS_TYPE_DESKTOP_WINDOW,
+						       NautilusDesktopWindowDetails);
 }
 
 static void
@@ -132,14 +233,16 @@ nautilus_desktop_window_screen_size_changed (GdkScreen             *screen,
 		      NULL);
 }
 
-NautilusDesktopWindow *
-nautilus_desktop_window_new (GtkApplication *application,
-			     GdkScreen      *screen)
+static NautilusDesktopWindow *
+nautilus_desktop_window_new (void)
 {
+	GdkScreen *screen;
+	GApplication *application;
 	NautilusDesktopWindow *window;
 	int width_request, height_request;
-        GdkRGBA transparent = {0, 0, 0, 0};
 
+	application = g_application_get_default ();
+	screen = gdk_screen_get_default ();
 	width_request = gdk_screen_get_width (screen);
 	height_request = gdk_screen_get_height (screen);
 
@@ -148,22 +251,31 @@ nautilus_desktop_window_new (GtkApplication *application,
 			       "disable-chrome", TRUE,
 			       "width_request", width_request,
 			       "height_request", height_request,
-			       "screen", screen,
 			       NULL);
 
-	/* Special sawmill setting*/
-	gtk_window_set_wmclass (GTK_WINDOW (window), "desktop_window", "Nautilus");
-
-	/* Point window at the desktop folder.
-	 * Note that nautilus_desktop_window_init is too early to do this.
-	 */
-	nautilus_desktop_window_update_directory (window);
-        gtk_widget_override_background_color (GTK_WIDGET (window), 0, &transparent);
-
-
 	return window;
 }
 
+static NautilusDesktopWindow *the_desktop_window = NULL;
+
+void
+nautilus_desktop_window_ensure (void)
+{
+	NautilusDesktopWindow *window;
+
+	if (!the_desktop_window) {
+		window = nautilus_desktop_window_new ();
+		g_object_add_weak_pointer (G_OBJECT (window), (gpointer *) &the_desktop_window);
+		the_desktop_window = window;
+	}
+}
+
+GtkWidget *
+nautilus_desktop_window_get (void)
+{
+	return GTK_WIDGET (the_desktop_window);
+}
+
 static gboolean
 nautilus_desktop_window_delete_event (GtkWidget *widget,
 				      GdkEventAny *event)
@@ -195,6 +307,8 @@ unrealize (GtkWidget *widget)
 		details->size_changed_id = 0;
 	}
 
+	gtk_widget_destroy (details->desktop_selection);
+
 	GTK_WIDGET_CLASS (nautilus_desktop_window_parent_class)->unrealize (widget);
 }
 
@@ -264,6 +378,7 @@ nautilus_desktop_window_class_init (NautilusDesktopWindowClass *klass)
 	GObjectClass *oclass = G_OBJECT_CLASS (klass);
 
 	oclass->constructed = nautilus_desktop_window_constructed;
+	oclass->finalize = nautilus_desktop_window_finalize;
 
 	wclass->realize = realize;
 	wclass->unrealize = unrealize;
