@@ -53,8 +53,6 @@ struct _NautilusToolbarPrivate {
 
 	guint popup_timeout_id;
 
-	gdouble scale_zoom_level;
-
 	GtkWidget *view_button;
 	GtkWidget *action_button;
 
@@ -65,9 +63,7 @@ struct _NautilusToolbarPrivate {
 	GtkWidget *visible_columns;
 	GtkWidget *stop;
 	GtkWidget *reload;
-	GtkAdjustment *zoom_adjustment_grid;
-	GtkAdjustment *zoom_adjustment_list;
-	GtkAdjustment *active_zoom_adjustment;
+	GtkAdjustment *zoom_adjustment;
 	GtkWidget *zoom_level_scale;
 	GMenu *action_menu;
 
@@ -87,32 +83,6 @@ G_DEFINE_TYPE_WITH_PRIVATE(NautilusToolbar, nautilus_toolbar, GTK_TYPE_HEADER_BA
 
 static void unschedule_menu_popup_timeout (NautilusToolbar *self);
 
-void
-nautilus_toolbar_update_view_mode (NautilusToolbar *self,
-				   const gchar *view_mode)
-{
-	gchar *name;
-	GtkWidget *image;
-
-	if (g_strcmp0 (view_mode, "list") == 0) {
-		name = "view-list-symbolic";
-		self->priv->active_zoom_adjustment = self->priv->zoom_adjustment_list;
-	} else if (g_strcmp0 (view_mode, "grid") == 0) {
-		name = "view-grid-symbolic";
-		self->priv->active_zoom_adjustment = self->priv->zoom_adjustment_grid;
-	} else {
-		g_assert_not_reached ();
-	}
-
-	gtk_range_set_adjustment (GTK_RANGE (self->priv->zoom_level_scale),
-				  self->priv->active_zoom_adjustment);
-
-	image = gtk_image_new ();
-	gtk_button_set_image (GTK_BUTTON (self->priv->view_button), image);
-	gtk_image_set_from_icon_name (GTK_IMAGE (image), name,
-				      GTK_ICON_SIZE_MENU);
-}
-
 static void
 toolbar_update_appearance (NautilusToolbar *self)
 {
@@ -154,25 +124,6 @@ activate_forward_menu_item_callback (GtkMenuItem *menu_item, NautilusWindow *win
 	activate_back_or_forward_menu_item (menu_item, window, FALSE);
 }
 
-void
-nautilus_toolbar_sync_navigation_buttons (NautilusToolbar *self)
-{
-	NautilusWindowSlot *active_slot;
-	GAction *action;
-	gboolean enabled;
-
-	/* Check if the back and forward buttons need enabling or disabling. */
-	active_slot = nautilus_window_get_active_slot (self->priv->window);
-
-	action = g_action_map_lookup_action (G_ACTION_MAP (self->priv->window), "back");
-	enabled = nautilus_window_slot_get_back_history (active_slot) != NULL;
-	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-
-	action = g_action_map_lookup_action (G_ACTION_MAP (self->priv->window), "forward");
-	enabled = nautilus_window_slot_get_forward_history (active_slot) != NULL;
-	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-}
-
 static void
 fill_menu (NautilusWindow *window,
 	   GtkWidget *menu,
@@ -293,6 +244,66 @@ show_menu (NautilusToolbar *self,
                         button, event_time);
 }
 
+static void
+action_view_mode_state_changed (GActionGroup *action_group,
+				gchar *action_name,
+				GVariant *value,
+				gpointer user_data)
+{
+	NautilusToolbar *self = user_data;
+	const gchar *view_mode = g_variant_get_string (value, NULL);
+	const gchar *name;
+	GtkWidget *image;
+
+	if (g_strcmp0 (view_mode, "list") == 0) {
+		name = "view-list-symbolic";
+	} else if (g_strcmp0 (view_mode, "grid") == 0) {
+		name = "view-grid-symbolic";
+	} else {
+		g_assert_not_reached ();
+	}
+
+	image = gtk_image_new ();
+	gtk_button_set_image (GTK_BUTTON (self->priv->view_button), image);
+	gtk_image_set_from_icon_name (GTK_IMAGE (image), name,
+				      GTK_ICON_SIZE_MENU);
+}
+
+static void
+action_reload_enabled_changed (GActionGroup *action_group,
+			       gchar *action_name,
+			       gboolean enabled,
+			       gpointer user_data)
+{
+	NautilusToolbar *self = user_data;
+	gtk_widget_set_visible (self->priv->reload, enabled);
+}
+
+static void
+action_stop_enabled_changed (GActionGroup *action_group,
+			     gchar *action_name,
+			     gboolean enabled,
+			     gpointer user_data)
+{
+	NautilusToolbar *self = user_data;
+	gtk_widget_set_visible (self->priv->stop, enabled);
+}
+
+static void
+nautilus_toolbar_set_window (NautilusToolbar *self,
+			     NautilusWindow *window)
+
+{
+	self->priv->window = window;
+
+	g_signal_connect (self->priv->window, "action-enabled-changed::stop",
+			  G_CALLBACK (action_stop_enabled_changed), self);
+	g_signal_connect (self->priv->window, "action-enabled-changed::reload",
+			  G_CALLBACK (action_reload_enabled_changed), self);
+	g_signal_connect (self->priv->window, "action-state-changed::view-mode",
+			  G_CALLBACK (action_view_mode_state_changed), self);
+}
+
 #define MENU_POPUP_TIMEOUT 1200
 
 typedef struct {
@@ -393,6 +404,19 @@ zoom_level_changed (GtkRange *range,
 					    g_variant_new_int32 ((gint) zoom_level));
 }
 
+static void
+view_menu_popover_closed (GtkPopover *popover,
+			  NautilusToolbar *self)
+{
+	NautilusWindowSlot *slot;
+	NautilusView *view;
+
+	slot = nautilus_window_get_active_slot (self->priv->window);
+	view = nautilus_window_slot_get_current_view (slot);
+
+	nautilus_view_grab_focus (view);
+}
+
 static void
 nautilus_toolbar_init (NautilusToolbar *self)
 {
@@ -412,8 +436,7 @@ nautilus_toolbar_init (NautilusToolbar *self)
 	builder = gtk_builder_new_from_resource ("/org/gnome/nautilus/nautilus-toolbar-view-menu.xml");
 	self->priv->view_menu_widget =  GTK_WIDGET (gtk_builder_get_object (builder, "view_menu_widget"));
 	self->priv->zoom_level_scale = GTK_WIDGET (gtk_builder_get_object (builder, "zoom_level_scale"));
-	self->priv->zoom_adjustment_grid = g_object_ref (GTK_ADJUSTMENT (gtk_builder_get_object (builder, "zoom_adjustment_grid")));
-	self->priv->zoom_adjustment_list = g_object_ref (GTK_ADJUSTMENT (gtk_builder_get_object (builder, "zoom_adjustment_list")));
+	self->priv->zoom_adjustment = GTK_ADJUSTMENT (gtk_builder_get_object (builder, "zoom_adjustment"));
 
 	self->priv->sort_menu =  GTK_WIDGET (gtk_builder_get_object (builder, "sort_menu"));
 	self->priv->sort_trash_time =  GTK_WIDGET (gtk_builder_get_object (builder, "sort_trash_time"));
@@ -422,6 +445,8 @@ nautilus_toolbar_init (NautilusToolbar *self)
 	self->priv->reload =  GTK_WIDGET (gtk_builder_get_object (builder, "reload"));
 	self->priv->stop =  GTK_WIDGET (gtk_builder_get_object (builder, "stop"));
 
+	g_signal_connect (self->priv->view_menu_widget, "closed",
+			  G_CALLBACK (view_menu_popover_closed), self);
 	gtk_menu_button_set_popover (GTK_MENU_BUTTON (self->priv->view_button),
 				     self->priv->view_menu_widget);
 	g_object_unref (builder);
@@ -432,10 +457,6 @@ nautilus_toolbar_init (NautilusToolbar *self)
 					G_MENU_MODEL (self->priv->action_menu));
 	g_object_unref (builder);
 
-	g_signal_connect(self->priv->zoom_level_scale, "value-changed",
-			 G_CALLBACK(zoom_level_changed),
-			 self);
-
 	g_object_set_data (G_OBJECT (self->priv->back_button), "nav-direction",
 			   GUINT_TO_POINTER (NAUTILUS_NAVIGATION_DIRECTION_BACK));
 	g_object_set_data (G_OBJECT (self->priv->forward_button), "nav-direction",
@@ -448,6 +469,8 @@ nautilus_toolbar_init (NautilusToolbar *self)
 			  G_CALLBACK (navigation_button_press_cb), self);
 	g_signal_connect (self->priv->forward_button, "button-release-event",
 			  G_CALLBACK (navigation_button_release_cb), self);
+	g_signal_connect (self->priv->zoom_level_scale, "value-changed",
+			  G_CALLBACK (zoom_level_changed), self);
 
 	gtk_widget_show_all (GTK_WIDGET (self));
 	toolbar_update_appearance (self);
@@ -481,7 +504,7 @@ nautilus_toolbar_set_property (GObject *object,
 
 	switch (property_id) {
 	case PROP_WINDOW:
-		self->priv->window = g_value_get_object (value);
+		nautilus_toolbar_set_window (self, g_value_get_object (value));
 		break;
 	case PROP_SHOW_LOCATION_ENTRY:
 		nautilus_toolbar_set_show_location_entry (self, g_value_get_boolean (value));
@@ -543,32 +566,16 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
 	gtk_widget_class_bind_template_child_private (widget_class, NautilusToolbar, forward_button);
 }
 
-void
-nautilus_toolbar_action_menu_add_item (NautilusToolbar *self,
-				       GMenuItem       *item,
-				       const gchar     *section_name)
-{
-	nautilus_gmenu_add_item_in_submodel (self->priv->action_menu,
-					     item,
-					     section_name,
-					     FALSE);
-}
-
 void
 nautilus_toolbar_reset_menus (NautilusToolbar *self)
 {
 	NautilusWindowSlot *slot;
 	NautilusView *view;
 	GActionGroup *view_action_group;
-	GtkBuilder *builder;
-
-	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (self->priv->action_button), NULL);
-
-	builder = gtk_builder_new_from_resource ("/org/gnome/nautilus/nautilus-toolbar-action-menu.xml");
-	self->priv->action_menu = G_MENU (gtk_builder_get_object (builder, "action-menu"));
-	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (self->priv->action_button),
-					G_MENU_MODEL (self->priv->action_menu));
-	g_object_unref (builder);
+	GVariant *variant;
+	GVariantIter iter;
+	gboolean sort_trash, sort_search, has_sort;
+	const gchar *hint;
 
 	/* Allow actions from the current view to be activated through
 	 * the view menu and action menu of the toolbar */
@@ -579,76 +586,34 @@ nautilus_toolbar_reset_menus (NautilusToolbar *self)
 					"view",
 					G_ACTION_GROUP (view_action_group));
 
-	gtk_widget_hide (self->priv->sort_menu);
-	gtk_widget_hide (self->priv->sort_trash_time);
-	gtk_widget_hide (self->priv->sort_search_relevance);
-	gtk_widget_hide (self->priv->visible_columns);
-}
+	gtk_widget_set_visible (self->priv->visible_columns,
+				g_action_group_has_action (view_action_group, "visible-columns"));
 
-void
-nautilus_toolbar_show_sort_menu (NautilusToolbar *self)
-{
-	gtk_widget_show (self->priv->sort_menu);
-}
+	has_sort = g_action_group_has_action (view_action_group, "sort");
+	sort_trash = sort_search = FALSE;
+	gtk_widget_set_visible (self->priv->sort_menu, has_sort);
 
-void
-nautilus_toolbar_show_sort_trash_time (NautilusToolbar *self)
-{
-	gtk_widget_show (self->priv->sort_trash_time);
-}
+	if (has_sort) {
+		variant = g_action_group_get_action_state_hint (view_action_group, "sort");
+		g_variant_iter_init (&iter, variant);
 
-void
-nautilus_toolbar_show_sort_search_relevance (NautilusToolbar *self)
-{
-	gtk_widget_show (self->priv->sort_search_relevance);
-}
+		while (g_variant_iter_next (&iter, "&s", &hint)) {
+			if (g_strcmp0 (hint, "trash-time") == 0)
+				sort_trash = TRUE;
+			if (g_strcmp0 (hint, "search-relevance") == 0)
+				sort_search = TRUE;
+		}
 
-void
-nautilus_toolbar_show_visible_columns (NautilusToolbar *self)
-{
-	gtk_widget_show (self->priv->visible_columns);
-}
+		g_variant_unref (variant);
+	}
 
-void
-nautilus_toolbar_show_stop (NautilusToolbar *self)
-{
-	gtk_widget_show (self->priv->stop);
-}
+	gtk_widget_set_visible (self->priv->sort_trash_time, sort_trash);
+	gtk_widget_set_visible (self->priv->sort_search_relevance, sort_search);
 
-void
-nautilus_toolbar_show_reload (NautilusToolbar *self)
-{
-	gtk_widget_show (self->priv->reload);
-}
-
-void
-nautilus_toolbar_hide_stop (NautilusToolbar *self)
-{
-	gtk_widget_hide (self->priv->stop);
-}
-
-void
-nautilus_toolbar_hide_reload (NautilusToolbar *self)
-{
-	gtk_widget_hide (self->priv->reload);
-}
-
-void
-nautilus_toolbar_view_menu_widget_set_zoom_level (NautilusToolbar *self,
-						  gdouble          level)
-{
-	g_return_if_fail (NAUTILUS_IS_TOOLBAR (self));
-
-	/* We only want to change the level when there's and actual view
-	 * mode set, so skip all other calls to here. Those calls came from
-	 * update_toolbar_menus in the natuilus-view after a parent is set, etc.
-	 * We will receive eventually a mode change and another update here by nautilus view
-	 * update_toolbar_menus */
-	if (self->priv->active_zoom_adjustment == NULL)
-		return;
-
-	gtk_adjustment_set_value (GTK_ADJUSTMENT (self->priv->active_zoom_adjustment),
-				  level);
+	variant = g_action_group_get_action_state (view_action_group, "zoom-to-level");
+	gtk_adjustment_set_value (self->priv->zoom_adjustment,
+				  g_variant_get_int32 (variant));
+	g_variant_unref (variant);
 }
 
 GtkWidget *
@@ -662,6 +627,12 @@ nautilus_toolbar_new (NautilusWindow *window)
 			     NULL);
 }
 
+GMenu *
+nautilus_toolbar_get_action_menu (NautilusToolbar *self)
+{
+	return self->priv->action_menu;
+}
+
 GtkWidget *
 nautilus_toolbar_get_path_bar (NautilusToolbar *self)
 {
