@@ -30,7 +30,6 @@
 #include "nautilus-floating-bar.h"
 #include "nautilus-list-view.h"
 #include "nautilus-special-location-bar.h"
-#include "nautilus-toolbar.h"
 #include "nautilus-trash-bar.h"
 #include "nautilus-window-private.h"
 #include "nautilus-x-content-bar.h"
@@ -2325,7 +2324,6 @@ nautilus_window_slot_connect_new_content_view (NautilusWindowSlot *slot)
 static void
 nautilus_window_slot_switch_new_content_view (NautilusWindowSlot *slot)
 {
-	NautilusWindow *window;
 	GtkWidget *widget;
 
 	if ((slot->details->new_content_view == NULL) ||
@@ -2333,8 +2331,6 @@ nautilus_window_slot_switch_new_content_view (NautilusWindowSlot *slot)
 		return;
 	}
 
-	window = nautilus_window_slot_get_window (slot);
-
 	if (slot->details->content_view != NULL) {
 		widget = GTK_WIDGET (slot->details->content_view);
 		gtk_widget_destroy (widget);
@@ -2349,11 +2345,6 @@ nautilus_window_slot_switch_new_content_view (NautilusWindowSlot *slot)
 		widget = GTK_WIDGET (slot->details->content_view);
 		gtk_container_add (GTK_CONTAINER (slot->details->view_overlay), widget);
 		gtk_widget_show (widget);
-
-		if (!NAUTILUS_IS_SEARCH_DIRECTORY (nautilus_view_get_model (slot->details->content_view)) &&
-		    slot == nautilus_window_get_active_slot (window)) {
-			nautilus_view_grab_focus (slot->details->content_view);
-		}
 	}
 }
 
