@@ -15,9 +15,7 @@
    General Public License for more details.
   
    You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   License along with this program; if not, see <http://www.gnu.org/licenses/>.
   
    Author: Christian Neumair <cneumair@gnome.org>
 */
@@ -125,5 +123,6 @@ void    nautilus_window_slot_go_up                         (NautilusWindowSlot *
 void    nautilus_window_slot_set_status			   (NautilusWindowSlot *slot,
 							    const char         *primary_status,
 							    const char         *detail_status);
+void nautilus_window_slot_sync_view_mode (NautilusWindowSlot *slot);
 
 #endif /* NAUTILUS_WINDOW_SLOT_H */
