@@ -42,6 +42,8 @@
 #include "nautilus-window-slot.h"
 #include "nautilus-list-view.h"
 #include "nautilus-view.h"
+#include "nautilus-notification-manager.h"
+#include "nautilus-notification-delete.h"
 
 #include <eel/eel-debug.h>
 #include <eel/eel-gtk-extensions.h>
@@ -58,6 +60,7 @@
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-file-operations.h>
+#include <libnautilus-private/nautilus-file-undo-manager.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-metadata.h>
 #include <libnautilus-private/nautilus-profile.h>
@@ -81,6 +84,7 @@ static void mouse_back_button_changed		     (gpointer                  callback_
 static void mouse_forward_button_changed	     (gpointer                  callback_data);
 static void use_extra_mouse_buttons_changed          (gpointer                  callback_data);
 static void nautilus_window_initialize_actions 	     (NautilusWindow *window);
+static GtkWidget * nautilus_window_ensure_location_entry (NautilusWindow *window);
 
 /* Sanity check: highest mouse button value I could find was 14. 5 is our 
  * lower threshold (well-documented to be the one of the button events for the 
@@ -252,8 +256,9 @@ action_enter_location (GSimpleAction *action,
 		       GVariant      *state,
 		       gpointer       user_data)
 {
-	g_action_group_activate_action (G_ACTION_GROUP (g_application_get_default ()),
-					"enter-location", NULL);
+	NautilusWindow *window = user_data;
+
+	nautilus_window_ensure_location_entry (window);
 }
 
 static void
@@ -351,6 +356,26 @@ action_prompt_for_location_home (GSimpleAction *action,
 	gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 }
 
+static void
+action_redo (GSimpleAction *action,
+	     GVariant      *state,
+	     gpointer       user_data)
+{
+	NautilusWindow *window = user_data;
+
+	nautilus_file_undo_manager_redo (GTK_WINDOW (window));
+}
+
+static void
+action_undo (GSimpleAction *action,
+	     GVariant      *state,
+	     gpointer       user_data)
+{
+	NautilusWindow *window = user_data;
+
+	nautilus_file_undo_manager_undo (GTK_WINDOW (window));
+}
+
 static void
 action_toggle_state_action_button (GSimpleAction *action,
 				   GVariant      *state,
@@ -371,11 +396,9 @@ action_view_mode (GSimpleAction *action,
 {
 	const gchar *name;
 	NautilusWindowSlot *slot;
-	NautilusToolbar *toolbar;
 
 	name =  g_variant_get_string (value, NULL);
 	slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (user_data));
-	toolbar = NAUTILUS_TOOLBAR (nautilus_window_get_toolbar (NAUTILUS_WINDOW (user_data)));
 
 	if (g_strcmp0 (name, "list") == 0) {
 		nautilus_window_slot_set_content_view (slot, NAUTILUS_LIST_VIEW_ID);
@@ -385,11 +408,58 @@ action_view_mode (GSimpleAction *action,
 		g_assert_not_reached ();
 	}
 
-	nautilus_toolbar_update_view_mode (toolbar, name);
-
 	g_simple_action_set_state (action, value);
 }
 
+static void
+undo_manager_changed (NautilusWindow *window)
+{
+	NautilusToolbar *toolbar;
+	NautilusFileUndoInfo *info;
+	NautilusFileUndoManagerState undo_state;
+	gboolean undo_active, redo_active;
+	gchar *undo_label, *undo_description, *redo_label, *redo_description;
+	gboolean is_undo;
+	GMenu* undo_section;
+	GAction *action;
+
+	toolbar = NAUTILUS_TOOLBAR (window->details->toolbar);
+	undo_label = undo_description = redo_label = redo_description = NULL;
+
+	info = nautilus_file_undo_manager_get_action ();
+	undo_state = nautilus_file_undo_manager_get_state ();
+	undo_active = redo_active = FALSE;
+	if (info != NULL &&
+	    (undo_state > NAUTILUS_FILE_UNDO_MANAGER_STATE_NONE)) {
+		is_undo = (undo_state == NAUTILUS_FILE_UNDO_MANAGER_STATE_UNDO);
+		undo_active = is_undo;
+		redo_active = !is_undo;
+		nautilus_file_undo_info_get_strings (info,
+						     &undo_label, &undo_description,
+						     &redo_label, &redo_description);
+	}
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (window), "undo");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), undo_active);
+	action = g_action_map_lookup_action (G_ACTION_MAP (window), "redo");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), redo_active);
+
+	undo_section = g_menu_new ();
+	undo_label = undo_active ? undo_label : g_strdup (_("Undo"));
+	redo_label = redo_active ? redo_label : g_strdup (_("Redo"));
+	g_menu_append (undo_section, undo_label, "win.undo");
+	g_menu_append (undo_section, redo_label, "win.redo");
+	nautilus_gmenu_replace_section (nautilus_toolbar_get_action_menu (toolbar),
+					"undo-redo-section",
+					G_MENU_MODEL (undo_section));
+
+	g_object_unref (undo_section);
+	g_free (undo_label);
+	g_free (undo_description);
+	g_free (redo_label);
+	g_free (redo_description);
+}
+
 const GActionEntry win_entries[] = {
 	{ "back",  action_back },
 	{ "forward",  action_forward },
@@ -401,6 +471,8 @@ const GActionEntry win_entries[] = {
 	{ "enter-location", action_enter_location },
 	{ "bookmark-current-location", action_bookmark_current_location },
 	{ "toggle-search", NULL, NULL, "false", action_toggle_search },
+	{ "undo", action_undo },
+	{ "redo", action_redo },
 	{ "view-mode", NULL, "s", "''", action_view_mode },
 	/* Only accesible by shorcuts */
 	{ "close-current-view", action_close_current_view },
@@ -438,6 +510,8 @@ nautilus_window_initialize_actions (NautilusWindow *window)
 	nautilus_application_add_accelerator (app, "win.view-mode('grid')", "<control>2");
 	nautilus_application_add_accelerator (app, "win.close-current-view", "<control>w");
 	nautilus_application_add_accelerator (app, "win.reload", "<control>r");
+	nautilus_application_add_accelerator (app, "win.undo", "<control>z");
+	nautilus_application_add_accelerator (app, "win.redo", "<shift><control>z");
 	/* Only accesible by shorcuts */
 	nautilus_application_add_accelerator (app, "win.bookmark-current-location", "<control>d");
 	nautilus_application_add_accelerator (app, "win.up", "<alt>Up");
@@ -463,6 +537,10 @@ nautilus_window_initialize_actions (NautilusWindow *window)
 		nautilus_window_show_sidebar (window);
 
 	g_variant_unref (state);
+
+	g_signal_connect_object (nautilus_file_undo_manager_get (), "undo-changed",
+				 G_CALLBACK (undo_manager_changed), window, G_CONNECT_SWAPPED);
+	undo_manager_changed (window);
 }
 
 void
@@ -742,13 +820,6 @@ nautilus_window_sync_allow_stop (NautilusWindow *window,
 		if (slot_is_active) {
 			g_simple_action_set_enabled (G_SIMPLE_ACTION (stop_action), slot_allow_stop);
 			g_simple_action_set_enabled (G_SIMPLE_ACTION (reload_action), !slot_allow_stop);
-			if (slot_allow_stop) {
-				nautilus_toolbar_hide_reload (NAUTILUS_TOOLBAR (window->details->toolbar));
-				nautilus_toolbar_show_stop (NAUTILUS_TOOLBAR (window->details->toolbar));
-			} else {
-				nautilus_toolbar_hide_stop (NAUTILUS_TOOLBAR (window->details->toolbar));
-				nautilus_toolbar_show_reload (NAUTILUS_TOOLBAR (window->details->toolbar));
-			}
 		}
 		if (gtk_widget_get_realized (GTK_WIDGET (window))) {
 			update_cursor (window);
@@ -1347,6 +1418,8 @@ nautilus_window_sync_location_widgets (NautilusWindow *window)
 {
 	NautilusWindowSlot *slot;
 	GFile *location;
+	GAction *action;
+	gboolean enabled;
 
 	slot = window->details->active_slot;
 	location = nautilus_window_slot_get_location (slot);
@@ -1363,12 +1436,18 @@ nautilus_window_sync_location_widgets (NautilusWindow *window)
 		nautilus_path_bar_set_path (NAUTILUS_PATH_BAR (path_bar), location);
 	}
 
-	nautilus_toolbar_sync_navigation_buttons (NAUTILUS_TOOLBAR (window->details->toolbar));
+	action = g_action_map_lookup_action (G_ACTION_MAP (window), "back");
+	enabled = nautilus_window_slot_get_back_history (slot) != NULL;
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (window), "forward");
+	enabled = nautilus_window_slot_get_forward_history (slot) != NULL;
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
 
 	nautilus_window_sync_bookmarks (window);
 }
 
-GtkWidget *
+static GtkWidget *
 nautilus_window_ensure_location_entry (NautilusWindow *window)
 {
 	GtkWidget *location_entry;
@@ -1383,6 +1462,34 @@ nautilus_window_ensure_location_entry (NautilusWindow *window)
 	return location_entry;
 }
 
+static void
+nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
+                                 NautilusWindow          *window)
+{
+	NautilusNotificationDelete *notification;
+	NautilusFileUndoInfo *undo_info;
+	NautilusFileUndoManagerState state;
+	GList *files;
+
+	nautilus_notification_manager_remove_all (NAUTILUS_NOTIFICATION_MANAGER (window->details->notification_manager));
+	undo_info = nautilus_file_undo_manager_get_action ();
+	state = nautilus_file_undo_manager_get_state ();
+
+	if (undo_info != NULL &&
+            state == NAUTILUS_FILE_UNDO_MANAGER_STATE_UNDO &&
+            nautilus_file_undo_info_get_op_type (undo_info) == NAUTILUS_FILE_UNDO_OP_MOVE_TO_TRASH) {
+		files = nautilus_file_undo_info_trash_get_files (NAUTILUS_FILE_UNDO_INFO_TRASH (undo_info));
+
+		/* Don't pop up a notification if user canceled the operation */
+		if (g_list_length (files) > 0) {
+			notification = nautilus_notification_delete_new (window);
+			nautilus_notification_manager_add_notification (NAUTILUS_NOTIFICATION_MANAGER (window->details->notification_manager),
+		                                                        GTK_WIDGET (notification));
+		}
+		g_list_free (files);
+	}
+}
+
 static void
 path_bar_location_changed_callback (GtkWidget      *widget,
 				    GFile          *location,
@@ -1746,9 +1853,8 @@ create_notebook (NautilusWindow *window)
 	gtk_widget_show (notebook);
 	gtk_container_set_border_width (GTK_CONTAINER (notebook), 0);
 
-	gtk_box_pack_start (GTK_BOX (window->details->main_view),
-			    notebook,
-			    TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (window->details->main_view),
+                           notebook);
 
 	return notebook;
 }
@@ -1790,14 +1896,21 @@ nautilus_window_constructed (GObject *self)
 	gtk_container_add (GTK_CONTAINER (grid), window->details->content_paned);
 	gtk_widget_show (window->details->content_paned);
 
-	window->details->main_view = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-	gtk_paned_pack2 (GTK_PANED (window->details->content_paned), window->details->main_view,
-			 TRUE, FALSE);
+	window->details->main_view = gtk_overlay_new ();
+	gtk_paned_pack2 (GTK_PANED (window->details->content_paned),
+                         window->details->main_view,
+                         TRUE, FALSE);
 	gtk_widget_show (window->details->main_view);
 
 	window->details->notebook = create_notebook (window);
 	nautilus_window_set_initial_window_geometry (window);
 
+	window->details->notification_manager = GTK_WIDGET (nautilus_notification_manager_new ());
+	gtk_overlay_add_overlay (GTK_OVERLAY (window->details->main_view),
+                                 window->details->notification_manager);
+	g_signal_connect_after (nautilus_file_undo_manager_get (), "undo-changed",
+                                G_CALLBACK (nautilus_window_on_undo_changed), self);
+
 	/* Is required that the UI is constructed before initializating the actions, since
 	 * some actions trigger UI widgets to show/hide. */
 	nautilus_window_initialize_actions (window);
@@ -1922,6 +2035,10 @@ nautilus_window_finalize (GObject *object)
 		window->details->sidebar_width_handler_id = 0;
 	}
 
+	g_signal_handlers_disconnect_by_func (nautilus_file_undo_manager_get (),
+                                              G_CALLBACK (nautilus_window_on_undo_changed),
+                                              window);
+
 	/* nautilus_window_close() should have run */
 	g_assert (window->details->slots == NULL);
 
@@ -2261,11 +2378,6 @@ use_extra_mouse_buttons_changed (gpointer callback_data)
 	mouse_extra_buttons = g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_MOUSE_USE_EXTRA_BUTTONS);
 }
 
-
-/*
- * Main API
- */
-
 static void
 nautilus_window_init (NautilusWindow *window)
 {
